// testAnim.cxx

#include "testap3.h"

#define  ANIM_MAX    10

ACTIONANIM gActionAnim = NULL;
static TCHAR _s_szAnimBuf[256];
static int iAnim_max = ANIM_MAX;

typedef struct tagANIM {
   int   iDoneInit;
   int   iAnimCount;
   int   iAnimDir;
   int   iLastDir;
   RECT  rcRect;  // Original RECT
   RECT  rcCurrent;
   RECT  rcPrevious;
   RECT  rcMax;
   HANDLE hExisting;
   HANDLE hCurrent;
}ANIM, * PANIM;

static ANIM sAnim = {0};

void KillAnim( void )
{
   PANIM pan = &sAnim;
   if( pan->hCurrent )
      DeleteObject( pan->hCurrent );
   if( pan->hExisting )
      DeleteObject( pan->hExisting );
   ZeroMemory( pan, sizeof(ANIM) );
}

void ReplaceAnim( HWND hWnd )
{
   PANIM pan = &sAnim;
   if( pan->hExisting ) {
      HDC hdc = GetDC(hWnd);
      HDC hdcBits;
      HGDIOBJ old;
      if(!hdc)
         return;
      hdcBits = CreateCompatibleDC(hdc);
      if(!hdcBits)
         goto Clean_up;
      sprtf( "Replacing existing clip ...\n" );
      old = SelectObject( hdcBits, pan->hExisting );
      BitBlt( hdc,
         pan->rcMax.left,
         pan->rcMax.top,
         RECTWIDTH(&pan->rcMax),
         RECTHEIGHT(&pan->rcMax),
         hdcBits,
         0,
         0,
         SRCCOPY );
      SelectObject( hdcBits, old );
Clean_up:
      ReleaseDC(hWnd,hdc);
   }
}

void ActionAnim( HWND hWnd )
{
   int   increment = 0;
   if( m_bAnim && m_hBitmap && m_hDIB ) {
      // proceed to NEXT
      PANIM pan = &sAnim;
      HDC hdc = GetDC(hWnd);
      HDC hdcBits;
      HGDIOBJ old;
      if(!hdc)
         return;
      hdcBits = CreateCompatibleDC(hdc);
      if(!hdcBits)
         goto Clean_up;

      if( !pan->iDoneInit ) {
         pan->rcPrevious = pan->rcCurrent;   // keep PREVIOUS
         pan->rcMax = pan->rcRect;       // restart
         InflateRect( &pan->rcMax, iAnim_max, iAnim_max ); // bump to NEXT
         pan->hExisting = CropBitmap (m_hBitmap, NULL, &pan->rcMax, NULL);
         pan->hCurrent = CropBitmap (m_hBitmap, NULL, &pan->rcRect, NULL);
         pan->iDoneInit = 1;
      } else if( pan->iAnimDir != pan->iLastDir ) {
         ReplaceAnim( hWnd ); // put back original, if any ...
         pan->iLastDir = pan->iAnimDir;
      }
      if( pan->iAnimDir ) {
         // on DECREASE
         pan->iAnimCount--;
      } else {
         // on INCREASE
         pan->iAnimCount++;
      }
      increment = pan->iAnimCount;
      pan->rcPrevious = pan->rcCurrent;   // keep PREVIOUS
      pan->rcCurrent = pan->rcRect;       // restart
      InflateRect( &pan->rcCurrent, increment, increment ); // bump to NEXT
      if( pan->hCurrent ) {
         old = SelectObject( hdcBits, pan->hCurrent );
         StretchBlt( hdc,      // handle to destination DC
            pan->rcCurrent.left, //int nXOriginDest, // x-coord of destination upper-left corner
            pan->rcCurrent.top, //int nYOriginDest, // y-coord of destination upper-left corner
            RECTWIDTH(&pan->rcCurrent), // width of destination rectangle
            RECTHEIGHT(&pan->rcCurrent), // height of destination rectangle
            hdcBits,       // handle to source DC
            0,   // int nXOriginSrc,  // x-coord of source upper-left corner
            0,   // int nYOriginSrc,  // y-coord of source upper-left corner
            RECTWIDTH(&pan->rcRect), // width of source rectangle
            RECTHEIGHT(&pan->rcRect), // height of source rectangle
            SRCCOPY );  // DWORD dwRop       // raster operation code
         SelectObject( hdcBits, old );
      }
      DeleteDC(hdcBits);

      // establish NEXT cycle
      if( pan->iAnimDir ) {
         // on DECREASE
         if( (pan->iAnimCount - 1) <= 0 ) {
            pan->iAnimDir = 0;   // next is INCREASE
         }
      } else {
         // on INCREASE
         if( (pan->iAnimCount + 1) >= iAnim_max ) {
            pan->iAnimDir = 1;   // next is DECREASE
         }
      }
Clean_up:
      ReleaseDC(hWnd,hdc);
   }
}

PTSTR GetAnimString( void )
{
   PTSTR ps = _s_szAnimBuf;
   if( m_bAnim && m_hBitmap && m_hDIB ) {
      PANIM pan = &sAnim;
      wsprintf(ps, L"%d,%d,%d,%d %s",
         pan->rcCurrent.left,
         pan->rcCurrent.top,
         pan->rcCurrent.right,
         pan->rcCurrent.bottom,
         (pan->iAnimDir) ? L"dec" : L"inc" );
   } else {
      lstrcpy(ps, L"Off");
   }
   return ps;
}

void Do_IDM_ACTIONANIM( HWND hWnd )
{
   PRECT lprcClip = &m_Clip;
   BOOL anim = m_bAnim;   // extract
   anim = !anim;  // flip it
   sprtf( "Do_IDM_ACTIONANIM ... Going %s\n",
      (anim ? "ON" : "OFF") );
   if(anim) {  // if going ON
      if( !m_hBitmap || !m_hDIB || IsRectEmpty( lprcClip ) ) {
         anim = FALSE;
         sprtf( "Nothing to ANIMATE!\n" );
      } else {
         PANIM pan = &sAnim;
         KillAnim(); // delete any previous
         pan->iAnimCount = 0;
         pan->rcRect = *lprcClip;
         pan->rcCurrent = pan->rcRect;
         pan->iAnimDir = 0;   // on increase, 1 equal decrease
         pan->iDoneInit = 0;
         gActionAnim = ActionAnim;
      }
   } else {
      // OFF, or going OFF
      gActionAnim = NULL;  // clear the TIMER action
      ReplaceAnim( hWnd ); // put what was there, back
      KillAnim(); // and remove meory objects
   }
   m_bAnim = anim;

}

// eof - teatanim.cxx
