// testMouse.cxx

#include "testap3.h"

#define  MMIN_MARGX  4
#define  MMIN_MARGY  4
// Some useful macros.
#define MAX(a,b)     ((a) > (b) ? (a) : (b))
#define MIN(a,b)     ((a) < (b) ? (a) : (b))
#define SWAP(x,y)   ((x)^=(y)^=(x)^=(y))

//---------------------------------------------------------------------
//
// Function:   NormalizeRect
//
// Purpose:    Insure that the upper/left corner of the rectangle is
//             kept in rect.top/rect.right.  Swaps around coordinates
//             in the rectangle to make sure this is true.
//
//             Code was stolen verbatim from ShowDIB.
//
// Parms:      lpRect == Pointer to RECT to normalize.
//
// History:   Date      Reason
//             ????     Created
//             
//---------------------------------------------------------------------
// SWAP bits
#define  SWAP_LR     1
#define  SWAP_TB     2

int NormalizeRect (LPRECT lpRect)
{
   int   ret = 0;
   if (lpRect->right < lpRect->left) {
      SWAP (lpRect->right,lpRect->left);
      ret |= SWAP_LR;
   }

   if (lpRect->bottom < lpRect->top) {
      SWAP (lpRect->bottom,lpRect->top);
      ret |= SWAP_TB;
   }
   return ret;
}


void SetStatusMousePos( HWND hWnd, INT xPos, INT yPos )
{
   m_ptMouse.x = xPos;
   m_ptMouse.y = yPos;
   PaintStatusMousePos( hWnd );
}

void PaintStatusMousePos( HWND hWnd )
{
   TCHAR buff[256];
   PTSTR pb = buff;
   RECT  rc2 = m_Clip;  // get the CLIP
   int   len;
   INT xPos = m_ptMouse.x;
   INT yPos = m_ptMouse.y;

   if( m_bMakingClip ) {
      len = wsprintf(pb, L"Mouse %4d,%4d Making Clip %s", xPos, yPos,
         (m_FlagStretch ? L"Stretch" : L"") );
   } else if( IsRectEmpty( &rc2 ) ) {
      len = wsprintf(pb, L"Mouse %4d,%4d NO Clip %s", xPos, yPos,
         (m_FlagStretch ? L"Stretch" : L"") );
   } else {
      len = wsprintf(pb, L"Mouse %4d,%4d Clip(%d,%d,%d,%d) %s", xPos, yPos,
         rc2.left, rc2.top, rc2.right, rc2.bottom,
         (m_FlagStretch ? L"Stretch" : L"") );
   }

   if( m_hStatusWnd ) {
      Set_StatusText( pb );
   } else {
      RECT  rc;
      HDC   hdc;
      int   x, y;
      SIZE  sz;
      GetClientRect( hWnd, &rc );
      // write it to bottom right of window
      hdc = GetDC(hWnd);
      if(hdc) {
         GetTextExtentPoint(hdc, pb, len, &sz);
         x = rc.right -  sz.cx - MMIN_MARGX;
         y = rc.bottom - sz.cy - MMIN_MARGY;
         TextOut(hdc, x, y, pb, len ); 
         ReleaseDC(hWnd,hdc);
      }
   }
}


void Do_WM_MOUSEMOVE( HWND hWnd, WPARAM wParam, LPARAM lParam )
{
   INT   xPos = GET_X_LPARAM(lParam);
   INT   yPos = GET_Y_LPARAM(lParam);
   SetStatusMousePos( hWnd, xPos, yPos );
}

BOOL IsMouseMessage( UINT msg )
{
   BOOL bRet = FALSE;
   if(( msg >= WM_MOUSEFIRST ) && ( msg <= WM_MOUSELAST ))
      bRet = TRUE;

   return bRet;
}

void TrackMouse( HWND hWnd, LPRECT lpClipRect, int cxDIB, int cyDIB)
{

	HDC   hDC;
	MSG   msg;
	POINT ptOrigin, ptStart;
	RECT  rcClient, rcLast;

	hDC = GetDC (hWnd);
	if( !hDC )
		return;

	SetCapture( hWnd );  // GRAB THE MOUSE!!!

	GetClientRect( hWnd, &rcClient );

	// Get mouse coordinates relative to origin of DIB.  Then
	//  setup the clip rectangle accordingly (it already should
	//  contain the starting point in its top/left).
	ptOrigin.x  = GetScrollPos( hWnd, SB_HORZ );
	ptOrigin.y  = GetScrollPos( hWnd, SB_VERT );

   // start with ONE POINT
	lpClipRect->left  += ptOrigin.x;
	lpClipRect->top   += ptOrigin.y;
	lpClipRect->right  = lpClipRect->left;
	lpClipRect->bottom = lpClipRect->top;

	ptStart.x          = lpClipRect->left;    // Need to remember the
	ptStart.y          = lpClipRect->top;     //  starting point.

   // Display the starting coordinates.
	SetWindowOrgEx( hDC, ptOrigin.x, ptOrigin.y, NULL );

	// DRAW FIRST, but it will be a dot only
   DrawSelect( hDC, lpClipRect );

	rcLast = *lpClipRect;
	// Eat mouse messages until a WM_LBUTTONUP is encountered.
	// Meanwhile continue to draw a rubberbanding rectangle
	// and display it's dimensions
	for( ;; ) {
		// The WaitMessage function yields control to other threads
		// when a thread has no other messages in its message queue.
		// The WaitMessage function suspends the thread and does not
		// return until a new message is placed in the thread's
		// message queue. 
		WaitMessage();

		// PeekMessage Return Values
		// If a message is available, the return value is nonzero.
		if( PeekMessage( &msg, NULL, WM_MOUSEFIRST, WM_MOUSELAST, PM_REMOVE ) )
		{
         INT xPos = LOWORD (msg.lParam);
         INT yPos = HIWORD (msg.lParam);

			// Erase the OLD rectangle (if ANY)
         DrawSelect( hDC, lpClipRect );

			rcLast = *lpClipRect;

			// Determine new coordinates.
			lpClipRect->left   = ptStart.x;
			lpClipRect->top    = ptStart.y;

			lpClipRect->right  = xPos + ptOrigin.x;
			lpClipRect->bottom = yPos + ptOrigin.y;

			NormalizeRect( lpClipRect );

			// Keep the rectangle within the bounds of the DIB.
			lpClipRect->left   = MAX(lpClipRect->left,   0);
			lpClipRect->top    = MAX(lpClipRect->top,    0);
			lpClipRect->right  = MAX(lpClipRect->right,  0);
			lpClipRect->bottom = MAX(lpClipRect->bottom, 0);

			lpClipRect->left   = MIN(lpClipRect->left,   cxDIB);
			lpClipRect->top    = MIN(lpClipRect->top,    cyDIB);
			lpClipRect->right  = MIN(lpClipRect->right,  cxDIB);
			lpClipRect->bottom = MIN(lpClipRect->bottom, cyDIB);

			// Draw the NEW rectangle.
			DrawSelect( hDC, lpClipRect );

         SetStatusMousePos( hWnd, xPos, yPos );
			// If the button is released, quit.
			if( msg.message == WM_LBUTTONUP )
			{
				// we have completed tracking
				break;
			}
		}
	}	// FOREVER

	// Clean up.
	ReleaseCapture();

	ReleaseDC( hWnd, hDC );

} // end - void TrackMouse (HWND hWnd, LPRECT lpClipRect, int cxDIB, int cyDIB)

void Do_WM_LBUTTONDOWN( HWND hWnd, WPARAM wParam, LPARAM lParam )
{
   RECT  rc;
  	int   cxDIB, cyDIB;
   HDC   hdc;

   if(m_bAnim) Do_IDM_ACTIONANIM(hWnd);   // shut DOWN any ANIM

   GetClientRect( hWnd, &rc );
   cxDIB = rc.right;
   cyDIB = rc.bottom;
   if( m_hDIB && !m_FlagStretch ) {
      cxDIB = m_sDIBSize.cx;
      cyDIB = m_sDIBSize.cy;
   }

   // set and pass current mouse,
   // as new CLIP start
   rc.left = LOWORD (lParam);
   rc.top  = HIWORD (lParam);

   hdc = GetDC(hWnd);
   if( !hdc )
      return;

   SetWindowOrgEx( hdc,
		   GetScrollPos( hWnd, SB_HORZ ),
		   GetScrollPos( hWnd, SB_VERT ),
		   NULL );

   sprtf( "DrawSelect to remove any OLD ...\n" );
   DrawSelect( hdc, &m_Clip );   // remove any OLD
   if( m_hClipBMP ) {

      // only REPLACE if it is still the SAME RECT
      if( EqualRect( &m_rcBMPRect, &m_Clip ) ) {
         sprtf( "Replacing CLIP ...\n" );
         ReplaceClip( hdc, m_hClipBMP, &m_Clip );
      }

      DeleteObject(m_hClipBMP);
      m_hClipBMP = NULL;
   }

   ReleaseDC( hWnd, hdc );

   m_bMakingClip = TRUE;
   // ***********************************
   TrackMouse( hWnd, &rc, cxDIB, cyDIB );
   // ***********************************

   sprtf( "Updating CLIP from %s to %s ...\n",
      GetRectStg( &m_Clip ),
      GetRectStg( &rc ) );

   m_Clip = rc;   // update the GLOBAL clip region
   m_bSwishMode = gSwMode; // get COPY of current MODE

   bClipOnOff = FALSE;
   m_bMakingClip = FALSE;
   if( !IsRectEmpty( &rc ) ) {
      //HBITMAP hbmp = ScreenRect2Bitmap( hWnd, &rc );
      //if(hbmp) {
      //   WriteBMPFile( L"tempclip.bmp", hbmp );
      //   DeleteObject(hbmp);
      //}
   }
}

/* =============
   MOUSE MESSAGES
   from : C:\Program Files\Microsoft Platform SDK for Windows Server 2003 R2\Include\WinUser.h
#define WM_MOUSEFIRST                   0x0200
#define WM_MOUSEMOVE                    0x0200
#define WM_LBUTTONDOWN                  0x0201
#define WM_LBUTTONUP                    0x0202
#define WM_LBUTTONDBLCLK                0x0203
#define WM_RBUTTONDOWN                  0x0204
#define WM_RBUTTONUP                    0x0205
#define WM_RBUTTONDBLCLK                0x0206
#define WM_MBUTTONDOWN                  0x0207
#define WM_MBUTTONUP                    0x0208
#define WM_MBUTTONDBLCLK                0x0209
#if (_WIN32_WINNT >= 0x0400) || (_WIN32_WINDOWS > 0x0400)
#define WM_MOUSEWHEEL                   0x020A
#endif
#if (_WIN32_WINNT >= 0x0500)
#define WM_XBUTTONDOWN                  0x020B
#define WM_XBUTTONUP                    0x020C
#define WM_XBUTTONDBLCLK                0x020D
#endif
#if (_WIN32_WINNT >= 0x0500)
#define WM_MOUSELAST                    0x020D
#elif (_WIN32_WINNT >= 0x0400) || (_WIN32_WINDOWS > 0x0400)
#define WM_MOUSELAST                    0x020A
#else
#define WM_MOUSELAST                    0x0209
#endif /== (_WIN32_WINNT >= 0x0500) ==/

   ============= */
typedef struct tagU2STG {
   UINT  ui;
   PTSTR ps;
}U2STG, * PU2STG;

U2STG sMouseMsgs[] = {
   // #define WM_MOUSEFIRST                   0x0200
   { WM_MOUSEMOVE, L"WM_MOUSEMOVE(0x200)" },  // 0x0200
   { WM_LBUTTONDOWN, L"WM_LBUTTONDOWN(0x201)" }, // 0x0201
   { WM_LBUTTONUP, L"WM_LBUTTONUP(0x202)" }, //  0x0202
   { WM_LBUTTONDBLCLK, L"WM_LBUTTONDBLCLK(0x203)" }, // 0x0203
   { WM_RBUTTONDOWN, L"WM_RBUTTONDOWN(0x204)" }, // 0x0204
   { WM_RBUTTONUP, L"WM_RBUTTONUP(0x205)" }, //  0x0205
   { WM_RBUTTONDBLCLK, L"WM_RBUTTONDBLCLK(0x206)" }, // 0x0206
   { WM_MBUTTONDOWN, L"WM_MBUTTONDOWN(0x207)" }, // 0x0207
   { WM_MBUTTONUP, L"WM_MBUTTONUP(0x208)" }, // 0x0208
   { WM_MBUTTONDBLCLK, L"WM_MBUTTONDBLCLK(0x209)" }, // 0x0209
#if (_WIN32_WINNT >= 0x0400) || (_WIN32_WINDOWS > 0x0400)
   { WM_MOUSEWHEEL, L"WM_MOUSEWHEEL(0x20A)" }, // 0x020A
#endif
#if (_WIN32_WINNT >= 0x0500)
   { WM_XBUTTONDOWN, L"WM_XBUTTONDOWN(0x20B)" }, // 0x020B
   { WM_XBUTTONUP, L"WM_XBUTTONUP(0x20C)" }, //  0x020C
   { WM_XBUTTONDBLCLK, L"WM_XBUTTONDBLCLK(0x20D)" }, // 0x020D
#endif
   //#if (_WIN32_WINNT >= 0x0500)
   //#define WM_MOUSELAST                    0x020D
   //#elif (_WIN32_WINNT >= 0x0400) || (_WIN32_WINDOWS > 0x0400)
   //#define WM_MOUSELAST                    0x020A
   //#else
   //#define WM_MOUSELAST                    0x0209
   //#endif /== (_WIN32_WINNT >= 0x0500) ==/
   // LAST ENTRY
   { 0,                0  }
};

PTSTR GetMouseMsgName( UINT msg )
{
   static TCHAR _s_nomsg[256];
   PTSTR ps = _s_nomsg;
   PU2STG pus = &sMouseMsgs[0];
   while( pus->ps )
   {
      if( pus->ui == msg )
         return pus->ps;
      pus++;
   }
   wsprintf(ps, L"Unlisted %d", msg);
   return ps;
}

// eof - testMouse.cxx
